/* HelloUniverse.java
 * Revision of original by Sun Microsystems, Inc. and published in Prentice 
 * Hall's Java 3D API Jump-Start by Aaron E. Walsh and Doug Gehringer. See
 * Web3DBooks.com for details and original. */
import java.applet.Applet;
import java.awt.*;
import com.sun.j3d.utils.applet.MainFrame;
import com.sun.j3d.utils.geometry.ColorCube;
import com.sun.j3d.utils.universe.*;
import javax.media.j3d.*;
import javax.vecmath.*;

public class HelloUniverse extends Applet {
  private SimpleUniverse u = null;
  public BranchGroup createSceneGraph() {
	  BranchGroup objRoot = new BranchGroup(); // branch graph root
	  // Create TransformGroup node and initialize it to the identity then
	  // enable TRANSFORM_WRITE so our behavior code can modify it at run time:
	  TransformGroup objTrans = new TransformGroup();
	  objTrans.setCapability(TransformGroup.ALLOW_TRANSFORM_WRITE);
	  objRoot.addChild(objTrans); // add to root of subgraph
	  // Create a simple Shape3D node (ColorCube) and add it to the scene graph:
	  objTrans.addChild(new ColorCube(0.5));
	  // Create a new Behavior object to perform the rotation operation:
	  Alpha rot = new Alpha(5, 10000); // rotate 5 times...
	  Transform3D y = new Transform3D(); // ...on the y axis
	  RotationInterpolator rotator = new RotationInterpolator(rot, objTrans, y, 0f, 
                                   (float)Math.PI*2f); // the behavior (rotator)
	  BoundingSphere bounds = new BoundingSphere(new Point3d(0,0,0), 100);
	  rotator.setSchedulingBounds(bounds); // set behavior's scheduling region
	  objRoot.addChild(rotator); // add rotator behavior to scene graph
    objRoot.compile(); // compile scene graph (perform optomizations)
    return objRoot;
  }
  public void init() {
	  setLayout(new BorderLayout());
	  Canvas3D canvas = new Canvas3D(SimpleUniverse.getPreferredConfiguration());
	  add("Center", canvas);
	  BranchGroup scene = createSceneGraph(); // create a simple scene 
	  u = new SimpleUniverse(canvas); // attach it to the virtual universe
    // Move ViewPlatform back a bit so objects in scene can be viewed:
    u.getViewingPlatform().setNominalViewingTransform();
    u.addBranchGraph(scene);
  }
  // Enable program to run as both an application and applet:
  public static void main(String[] args) {
    new MainFrame(new HelloUniverse(), 500, 500); 
  }
  public HelloUniverse() { }
  public void destroy() { u.removeAllLocales(); }
}
