// Counter.h
#ifndef __COUNTER_H___
#define __COUNTER_H___

#include "stdafx.h"

// ----------------------------------------------------------------------------
// A simple counter class that keeps track of its minimum, maximum and
// accumulated value. We used DWORDs instead of doubles to save space and
// the expense of floating point arithmetic (at the expense of a little 
// bit of accuracy)
// ----------------------------------------------------------------------------
class Counter {
public:
	Counter() : dMin(0),dMax(0),accum(0) {}
	void Update( DWORD value );
	DWORD Average( DWORD count ) const;
	DWORD dMin;		// min value
	DWORD dMax;		// max value
	DWORD accum;	// accumulated value
};

// ----------------------------------------------------------------------------
// Updates counter, including setting new minimum and maximum if necessary
// ----------------------------------------------------------------------------
inline void Counter::Update( DWORD value )
{
	accum += value;
	if( dMin == 0 ) {
		dMin = value;
	} else {
		if( value < dMin ) {
			dMin = value; 
		} // if
	} // else
	if( value > dMax ) {
		dMax = value;
	} // if
}


// ----------------------------------------------------------------------------
// Calculates average value based on provided count, if we get zero because
// of imprecision of integer division, then we return dMin
// ----------------------------------------------------------------------------
inline DWORD Counter::Average( DWORD count ) const
{
	DWORD avg = accum/count; 
	if( avg == 0 ) {
		return dMin;
	} else {
		return avg;
	} // else
}


#endif // #ifndef __COUNTER_H___