// CritSect.h
#ifndef __CRITSECT_H__
#define __CRITSECT_H__

#include "stdafx.h"


// ----------------------------------------------------------------------------
// Simple critical section wrapper
// ----------------------------------------------------------------------------
class CritSect {
public:
	CritSect() 
	{
		::InitializeCriticalSection(&cs);
	}
	~CritSect()
	{
		::DeleteCriticalSection(&cs);
	}
	LPCRITICAL_SECTION operator &()
	{
		return &cs; 
	}
protected:
	CRITICAL_SECTION cs;
};


// ----------------------------------------------------------------------------
// Simple class which attempts to acquire critical section lock in constructor
// and releases it in destructor
// ----------------------------------------------------------------------------
class CritSectLock {
public:
	CritSectLock(LPCRITICAL_SECTION pcs) : m_pcs(pcs), locked(FALSE)
	{
		Lock();
	}
	~CritSectLock()
	{
		Unlock();
	}
	void Lock()
	{
		::EnterCriticalSection(m_pcs);
		locked = TRUE;
	}
	void Unlock()
	{
		// Calling LeaveCriticalSection when current thread doesn't own lock can cause
		// problems, hence the need for locked member var
		if( locked ) {
			::LeaveCriticalSection(m_pcs);
			locked = FALSE; 
		} // if
	}
protected:
	LPCRITICAL_SECTION m_pcs;
	BOOL locked;
};


#endif // #ifndef __CRITSECT_H__