// MonitorIsapiFilter.cpp : Defines the entry point for the DLL application.

#include "stdafx.h"
#pragma warning( disable : 4786 )  
#include <httpfilt.h>
#include <time.h>
#include "Timer.h"
#include "PageInfo.h"
#include "ContextInfo.h"
#include "Trace.h"

const double MIFVERSION = 1.01;
const int MAX_URL_LEN = 512; 

const DWORD NOTIFICATION_FLAGS = 
	SF_NOTIFY_URL_MAP | SF_NOTIFY_SEND_RESPONSE | SF_NOTIFY_SEND_RAW_DATA |			
	SF_NOTIFY_END_OF_REQUEST | SF_NOTIFY_ORDER_HIGH |		
	SF_NOTIFY_SECURE_PORT | SF_NOTIFY_NONSECURE_PORT; 

const LPCSTR PROFILE_DATA_FILENAME	= "c:\\mif-data%s.txt";

// Global data structures
PageMap pageMap; 
ContextMap contextMap;

// Critical section for saving state
CritSect gcs; 

// Global on/off switch. For performance reasons we won't bother
// to protect access to this var with a critical section. 
BOOL bOn = FALSE; 



// ----------------------------------------------------------------------------
// Dumps profile data in tab-delimited format to the browser.
// ----------------------------------------------------------------------------
DWORD DumpProfileData(
	PHTTP_FILTER_CONTEXT pfc  
)
{
	LPCSTR FIRSTLINE = "PageName\tNumCalls\tMinDuration\tMaxDuration\tAccumDuration\tAvgDuration\t"
						"MinBytes\tMaxBytes\tAccumBytes\tAvgBytes\tAvgBytesPerSec\n";

	LPCSTR HEADERS =	"HTTP/1.0 200 OK\r\n"
						"Content-Type: application/vnd.ms-excel\r\n"
						"Cache-Control: no-cache\r\n"
						"Expires: Mon, 09 Oct 2000 16:00:00 GMT\r\n" 
						"Pragma: no-cache\r\n"
						"\r\n";

	DWORD dwLen = strlen(HEADERS);
	pfc->WriteClient( pfc, (LPVOID)HEADERS, &dwLen, 0); 

	dwLen = strlen(FIRSTLINE);
	pfc->WriteClient( pfc, (LPVOID)FIRSTLINE, &dwLen, 0);

	strstream stream;
	pageMap.Dump(stream); 

	dwLen = stream.pcount();
	pfc->WriteClient( pfc, stream.str(), &dwLen, 0);

	return SF_STATUS_REQ_FINISHED;
}


// ----------------------------------------------------------------------------
// Saves profile data to local file. Returns TRUE on success
// ----------------------------------------------------------------------------
BOOL SaveProfileData(
	LPCSTR lpszFilename
)
{
	FILE* pFile = fopen( lpszFilename, "wb" ); 
	if( !pFile ) {
		return FALSE; 
	} // if

	strstream stream;
	pageMap.Dump(stream); 

	long len = stream.pcount();
	long bytes = fwrite( stream.str(), sizeof(char), len, pFile ); 
	fclose( pFile ); 

	return (bytes==len);
}


// ----------------------------------------------------------------------------
// Loads saved profile data from local file if available. 
// Returns TRUE on success
// ----------------------------------------------------------------------------
BOOL LoadProfileData(
	LPCSTR lpszFilename
)
{
	FILE* pFile = fopen( lpszFilename, "rb" ); 
	if( !pFile ) {
		return FALSE; 
	} // if

	const int BUFF_SIZE = 1024; 
	char szLine[BUFF_SIZE];

	int ctr = 1; 
	while( fgets( szLine, BUFF_SIZE, pFile ) != NULL ) {
		if( ! pageMap.LoadLine( szLine )) {
			TRACE2( "[MIF] INFO: problem loading file %s line #%d, skipping line...\n", 
					lpszFilename, ctr ); 
		} // if
		ctr++; 
	} // while

	BOOL bResult = feof(pFile);
	fclose( pFile ); 
	return bResult; 
}


// ----------------------------------------------------------------------------
// Returns URL of current request. Result may be empty string.
// ----------------------------------------------------------------------------
inline void GetRequestUrl( 
	PHTTP_FILTER_CONTEXT pfc,
	char* szPageName, 
	DWORD dwLen
)
{
	szPageName[0] = '\0'; 

	// Fetch URL server variable.
	pfc->GetServerVariable( pfc, "URL", szPageName, &dwLen );

	// Convert to lowercase
	for( char* p=szPageName; (p && *p); p++ ) {
		*p = (char)tolower(*p);
	} // for
}


// ----------------------------------------------------------------------------
// Returns TRUE if IP of connected user can be determined and the IP is in
// one of the nonroutable address ranges. 
// IP range: 
// 10.0.0.0        -   10.255.255.255  (10/8 prefix)
// 172.16.0.0      -   172.31.255.255  (172.16/12 prefix)
// 192.168.0.0     -   192.168.255.255 (192.168/16 prefix)
// ----------------------------------------------------------------------------
inline BOOL GetRemoteAddr( 
	PHTTP_FILTER_CONTEXT pfc, 
	LPSTR szIP,
	DWORD dwLen
)
{
	if( !szIP ) {
		return FALSE; 
	} // if

	szIP[0] = '\0'; 

	// Fetch URL server variable.
	if( ! pfc->GetServerVariable( pfc, "REMOTE_ADDR", szIP, &dwLen )) {
		TRACE0( "[MIF] INFO: failed to get IP address of remote client\n" ); 
		return FALSE; 
	} // if

	return TRUE; 
}


// ----------------------------------------------------------------------------
// Returns TRUE if HTTP request was a POST
// ----------------------------------------------------------------------------
inline BOOL IsPOST( 
	PHTTP_FILTER_CONTEXT pfc 
)
{
	char szMethod[64];
	DWORD dwLen = sizeof(szMethod);

	// Fetch URL server variable.
	if( ! pfc->GetServerVariable( pfc, "REQUEST_METHOD", szMethod, &dwLen )) {
		TRACE0( "[MIF] INFO: failed to get HTTP request method\n" ); 
		return FALSE; 
	} // if

	return (stricmp(szMethod,"post")==0);
}



// ----------------------------------------------------------------------------
// Returns TRUE if IP is in
// one of the nonroutable address ranges. 
// IP range: 
// 10.0.0.0        -   10.255.255.255  (10/8 prefix)
// 172.16.0.0      -   172.31.255.255  (172.16/12 prefix)
// 192.168.0.0     -   192.168.255.255 (192.168/16 prefix)
// and of course localhost
// 127.0.0.0	   -   127.255.255.255
// ----------------------------------------------------------------------------
inline BOOL IsClientLocal( 
	LPCSTR szIP
)
{
	if( !szIP ) {
		return FALSE; 
	} // if

	// Not the most efficient code in the world, but this is only called
	// when we are processing an internal command (which is rare)
	if( 
		// 10.0.0.0 - 10.255.255.255
		strncmp( szIP, "10.", 3) == 0 || 
		// 172.16.0.0 - 172.31.255.255
		strncmp( szIP, "172.16.", 7) == 0 || strncmp( szIP, "172.17.", 7) == 0 ||
		strncmp( szIP, "172.18.", 7) == 0 || strncmp( szIP, "172.19.", 7) == 0 ||
		strncmp( szIP, "172.20.", 7) == 0 || strncmp( szIP, "172.21.", 7) == 0 ||
		strncmp( szIP, "172.22.", 7) == 0 || strncmp( szIP, "172.23.", 7) == 0 ||
		strncmp( szIP, "172.24.", 7) == 0 || strncmp( szIP, "172.25.", 7) == 0 ||
		strncmp( szIP, "172.26.", 7) == 0 || strncmp( szIP, "172.27.", 7) == 0 ||
		strncmp( szIP, "172.28.", 7) == 0 || strncmp( szIP, "172.29.", 7) == 0 ||
		strncmp( szIP, "172.30.", 7) == 0 || strncmp( szIP, "172.31.", 7) == 0 ||
		// 192.16.0.0 - 192.168.255.255
		strncmp( szIP, "192.168.", 8) == 0 ||
		// 127.0.0.0 - 127.255.255.255
		strncmp( szIP, "127.", 4) == 0 ) {
		return TRUE;
	} // if

	return FALSE; 
}


// ----------------------------------------------------------------------------
// Disable future notifications for this request
// ----------------------------------------------------------------------------
inline void DisableNotifications( 
	PHTTP_FILTER_CONTEXT pfc
)
{
	pfc->ServerSupportFunction( pfc, SF_REQ_DISABLE_NOTIFICATIONS, 0, 
		NOTIFICATION_FLAGS, 0 );
}


// ----------------------------------------------------------------------------
// ----------------------------------------------------------------------------
void LoadFilterData()
{
	const int BUFF_SIZE = 512;
	char szFilename[BUFF_SIZE]; 
	
	sprintf(szFilename, PROFILE_DATA_FILENAME, "" ); 
	if( LoadProfileData( szFilename )) {
		TRACE1( "[MIF] INFO: Loaded previously saved profile data from '%s'\n", 
			szFilename ); 
		// Saved profile data is there, let's assume we should be turned on
		bOn = TRUE; 
	} else {
		TRACE1( "[MIF] INFO: Profile data file '%s' is missing or couldn't be loaded\n", 
			szFilename ); 
	} // else
}


// ----------------------------------------------------------------------------
// ----------------------------------------------------------------------------
void SaveFilterData( LPCSTR lpszSuffix )
{
	const int BUFF_SIZE = 512;
	char szFilename[BUFF_SIZE]; 

	sprintf(szFilename, PROFILE_DATA_FILENAME, lpszSuffix ); 
	if( SaveProfileData( szFilename )) {
		TRACE1( "[MIF] INFO: Saved profile data to '%s'\n", szFilename ); 
	} else {
		TRACE1( "[MIF] ERROR: Failed to save profile data to '%s'\n", szFilename ); 
	} // else
}



// ----------------------------------------------------------------------------
// Called once by IIS when our filter is loaded
// ----------------------------------------------------------------------------
BOOL WINAPI GetFilterVersion(
	PHTTP_FILTER_VERSION pVer  
)
{
	try {
		// Filter is "off" by default if we don't find any saved profile data
		bOn = FALSE;

		pVer->dwFilterVersion = HTTP_FILTER_REVISION;
		strncpy( pVer->lpszFilterDesc, "MonitorIsapiFilter", SF_MAX_FILTER_DESC_LEN-1 );
		pVer->dwFlags = NOTIFICATION_FLAGS;

		LoadFilterData(); 

		if( bOn ) {
			TRACE1( "[MIF] INFO: Filter version %1.2f loaded (currently ON)\n", MIFVERSION ); 
		} else {
			TRACE1( "[MIF] INFO: Filter version %1.2f loaded (currently OFF)\n", MIFVERSION ); 
		} // else

		return TRUE;

	} catch( ... ) {
		TRACE0( "[MIF] ERROR: unhandled exception in GetFilterVersion\n" );
		return FALSE; 
	} // catch
	
}



// ----------------------------------------------------------------------------
// ----------------------------------------------------------------------------
DWORD BadRequest(
	PHTTP_FILTER_CONTEXT pfc  
)
{
	char MSG[]	=	"HTTP/1.0 400 Bad Request\r\n"
					"Content-Type: text/plain\r\n"
					"Cache-Control: no-cache\r\n"
					"Expires: Mon, 09 Oct 2000 16:00:00 GMT\r\n" 
					"Pragma: no-cache\r\n"
					"\r\n"
					"Bad Request";

	DWORD dwLen = strlen(MSG);
	pfc->WriteClient( pfc, MSG, &dwLen, 0); 
	return SF_STATUS_REQ_FINISHED;
}


// ----------------------------------------------------------------------------
// ----------------------------------------------------------------------------
DWORD CommandHelp(
	PHTTP_FILTER_CONTEXT pfc  
)
{
	char MSG[]	=	
		"HTTP/1.0 200 OK\r\n"
		"Content-Type: text/html\r\n"
		"Cache-Control: no-cache\r\n"
		"Expires: Mon, 09 Oct 2000 16:00:00 GMT\r\n" 
		"Pragma: no-cache\r\n"
		"\r\n"
		"<html><body>"
		"Filter commands:<br>"
		"<a href=\"/monitor-isapi-filter-status\">monitor-isapi-filter-status</a> - displays filter status (on or off)<br>"
		"<a href=\"/monitor-isapi-filter-on\">monitor-isapi-filter-on</a> - turns filter on<br>"
		"<a href=\"/monitor-isapi-filter-off\">monitor-isapi-filter-off</a> - turns filter off<br>"
		"<a href=\"/monitor-isapi-filter-dump\">monitor-isapi-filter-dump</a> - dumps profile data by page<br>"
		"<form method=\"post\" action=\"/monitor-isapi-filter-reset\">"
		"<input type=\"submit\" value=\"Press to save data and reset counters\">"
		"</form>"
		"</body></html>";

	DWORD dwLen = strlen(MSG);
	pfc->WriteClient( pfc, MSG, &dwLen, 0); 
	return SF_STATUS_REQ_FINISHED;
}


// ----------------------------------------------------------------------------
// ----------------------------------------------------------------------------
void SaveDataAndResetCounters()
{
	CritSectLock lock(&gcs); // only allow one thread in here at a time

	const int BUFF_SIZE=64;
	char buff[BUFF_SIZE];
	time_t now = time(0);
	strftime( buff, BUFF_SIZE, "-%Y%m%d-%H%M%S", localtime( &now ) );
	SaveFilterData(buff);

	pageMap.Reset(); 
	contextMap.Reset();
	SaveFilterData("");
}



// ----------------------------------------------------------------------------
// Handle internal request (a command for MonitorIsapiFilter itself rather than
// a request for a standard URL)
// ----------------------------------------------------------------------------
DWORD OnInternalRequest(
	PHTTP_FILTER_CONTEXT pfc,  
	HTTP_FILTER_URL_MAP* pUrlMap
)
{
	DisableNotifications( pfc ); 

	const BUFF_SIZE=32;
	char szIP[BUFF_SIZE];

	if( !GetRemoteAddr( pfc, szIP, BUFF_SIZE )) {
		TRACE0( "[MIF] WARN: rejected command request because client IP couldn't be determined\n" );
		return BadRequest( pfc ); 
	} // if

	if( !IsClientLocal(szIP) ) {
		TRACE1( "[MIF] WARN: rejected command request from nonlocal client IP %s\n", szIP ); 
		return BadRequest( pfc ); 
	} // if

	if( strstr( pUrlMap->pszURL, "monitor-isapi-filter-dump" ) ) {
		TRACE1( "[MIF] INFO: got request to dump profile data from %s\n", szIP );
		return DumpProfileData( pfc ); 
	} // else

	char MSG_TEMPLATE[]="HTTP/1.0 200 OK\r\n"
						"Content-Type: text/plain\r\n"
						"Cache-Control: no-cache\r\n"
						"Expires: Mon, 09 Oct 2000 16:00:00 GMT\r\n" 
						"Pragma: no-cache\r\n"
						"\r\n"
						"%sMonitorIsapiFilter is %s";
	const int MSG_LEN=1024;
	char msg[MSG_LEN]; 

	if( strstr( pUrlMap->pszURL, "monitor-isapi-filter-on" ) ) {
		bOn = TRUE; 
		sprintf(msg,MSG_TEMPLATE, "", "ON");
		TRACE1( "[MIF] INFO: got request to set filter ON from %s\n", szIP );
	} else if( strstr( pUrlMap->pszURL, "monitor-isapi-filter-off" ) ) {
		bOn = FALSE; 
		sprintf(msg,MSG_TEMPLATE, "", "OFF");
		TRACE1( "[MIF] INFO: got request to set filter OFF from %s\n", szIP );
	} else if( strstr( pUrlMap->pszURL, "monitor-isapi-filter-status" ) ) {
		if( bOn ) {
			sprintf(msg,MSG_TEMPLATE, "", "ON");
			TRACE1( "[MIF] INFO: got status request from %s (filter is ON)\n", szIP );
		} else {
			sprintf(msg,MSG_TEMPLATE, "", "OFF");
			TRACE1( "[MIF] INFO: got status request from %s (filter is OFF)\n", szIP );
		} // else
	} else if( strstr( pUrlMap->pszURL, "monitor-isapi-filter-reset" ) && IsPOST(pfc) ) {
		SaveDataAndResetCounters();
		sprintf(msg,MSG_TEMPLATE, "Counters reset\r\n", "ON");
	} else {
		return CommandHelp(pfc); 
	} // else

	DWORD dwLen = strlen(msg);
	pfc->WriteClient( pfc, msg, &dwLen, 0); 

	return SF_STATUS_REQ_FINISHED;
}




// ----------------------------------------------------------------------------
// Handler for SF_NOTIFY_URL_MAP notification
// ----------------------------------------------------------------------------
DWORD OnNotifyUrlMap(
	PHTTP_FILTER_CONTEXT pfc,  
	HTTP_FILTER_URL_MAP* pUrlMap
)
{
	// If this is an internal request, handle it now
	if( strstr( pUrlMap->pszURL, "monitor-isapi-filter" ) ) {
		return OnInternalRequest( pfc, pUrlMap ); 
	} else if( !bOn ) { 
		// Otherwise, if we're turned off, disable notifications and exit
		DisableNotifications(pfc); 
		return SF_STATUS_REQ_NEXT_NOTIFICATION;
	} // if

	// Get context data structure for this request
	ContextInfo* pContextInfo = contextMap.GetOrCreateContextInfo( (DWORD) pfc );
	if( pContextInfo ) {
		// NOTE: no need to lock pContextInfo since it can only be active in one invocation
		// of HttpFilterProc at a time
		pContextInfo->start = StartTiming(); 
		pContextInfo->bytes = 0; 
	} else {
		TRACE0( "[MIF] INFO: failed to create context info in OnNotifyUrlMap\n" );
	} // else

	return SF_STATUS_REQ_NEXT_NOTIFICATION;
}


// ----------------------------------------------------------------------------
// Handler for SF_NOTIFY_SEND_RESPONSE
// ----------------------------------------------------------------------------
DWORD OnSendResponse( 
	PHTTP_FILTER_CONTEXT pfc,  
	HTTP_FILTER_SEND_RESPONSE* pfsr
)
{
	// The monitor SF_NOTIFY_SEND_RESPONSE to avoid a possible DoS attack
	// Since we allocate memory on each unique URI we serve, a malicious attacker
	// could hit server with random URIs, filling our memory. Therefore if we
	// see that this request resulted in a 404 Not Found error, we skip logging
	// for this request
	if( pfsr->HttpStatus == 404 ) {
		ContextInfo* pContextInfo = contextMap.GetContextInfo( (DWORD) pfc );
		if( pContextInfo ) {
			contextMap.FreeContextInfo( pContextInfo ); 
		} // if
		DisableNotifications( pfc ); 
	} // if
	return SF_STATUS_REQ_NEXT_NOTIFICATION;
}


// ----------------------------------------------------------------------------
// Handler for SF_NOTIFY_SEND_RAW_DATA notification
// ----------------------------------------------------------------------------
DWORD OnSendRawData(
	PHTTP_FILTER_CONTEXT pfc,  
	HTTP_FILTER_RAW_DATA* prd
)
{
	// Get context data structure for this request
	ContextInfo* pContextInfo = contextMap.GetContextInfo( (DWORD) pfc );
	if( pContextInfo ) {
		// NOTE: no need to lock pContextInfo since it can only be active in one invocation
		// of HttpFilterProc at a time
		pContextInfo->bytes += prd->cbInData;
	} else {
		// TRACE0( "[MIF] INFO: failed to get context info in OnSendRawData\n" );
	} // else

	return SF_STATUS_REQ_NEXT_NOTIFICATION;
}


// ----------------------------------------------------------------------------
// Handler for SF_NOTIFY_END_OF_REQUEST notification
// ----------------------------------------------------------------------------
DWORD OnEndOfRequest(
	PHTTP_FILTER_CONTEXT pfc  
)
{
	// Get context data structure for this request
	ContextInfo* pContextInfo = contextMap.GetContextInfo( (DWORD) pfc );
	if( pContextInfo ) {
		// NOTE: no need to lock pContextInfo since it can only be active in one invocation
		// of HttpFilterProc at a time
		
		// Get URL of request
		char szPageName[MAX_URL_LEN];
		GetRequestUrl( pfc, szPageName, MAX_URL_LEN );  

		// Get or create data structure for this page
		PageInfo* pPageInfo = pageMap.GetOrCreatePageInfo( szPageName );
		if( pPageInfo ) {
			CritSectLock lock(&(pPageInfo->cs)); // lock PageInfo structure
			DWORD dwDuration = (DWORD) EndTiming( pContextInfo->start ); 
			// If duration got truncated to 0, bump it up to minimum resolution of 1ms
			if( dwDuration == 0 ) {
				dwDuration = 1;
			} // if
			DWORD dwBytes = pContextInfo->bytes;

			pPageInfo->numcalls++;
			pPageInfo->duration.Update(dwDuration);
			pPageInfo->bytes.Update(dwBytes);
			// Make copies of pPageInfo data so we can exit critsect before
			// expensive sprintf/trace calls
			long numcalls = pPageInfo->numcalls;
			Counter duration = pPageInfo->duration;
			Counter bytes = pPageInfo->bytes;

			// Release page and context info, unblocking waiting threads 
			lock.Unlock();
			contextMap.FreeContextInfo(pContextInfo);

			TRACE10( "[MIF] %s: calls=%lu, time=%lu (min=%lu, max=%lu, avg=%lu), bytes=%lu (min=%lu, max=%lu, avg=%lu)\n", 
				szPageName, numcalls, 
				dwDuration, duration.dMin, duration.dMax, duration.Average(numcalls),
				dwBytes, bytes.dMin, bytes.dMax, bytes.Average(numcalls));

		} else {
			TRACE0( "[MIF] INFO: failed to get or create page info in OnEndOfRequest\n" );
		} // else

	} else {
		// TRACE0( "[MIF] INFO: failed to get context info in OnEndOfRequest\n" );
	} // else

	return SF_STATUS_REQ_NEXT_NOTIFICATION;
}


// ----------------------------------------------------------------------------
// Main filter entry point
// ----------------------------------------------------------------------------
DWORD WINAPI HttpFilterProc(
	PHTTP_FILTER_CONTEXT pfc,  
	DWORD notificationType,  
	LPVOID pvNotification  
)
{
	// Uncomment to see how much overhead MonitorIsapiFilter itself is adding
	// It seems to add less than 10 *micro*seconds when bOn==FALSE and about half
	// a millisecond on average when bOn==TRUE
  	// PROFILESCOPE( "HttpFilterProc" )
	try {
		if (notificationType == SF_NOTIFY_URL_MAP ) {
			return OnNotifyUrlMap( pfc, (HTTP_FILTER_URL_MAP*)pvNotification );
		} else if( notificationType == SF_NOTIFY_SEND_RESPONSE ) {
			return OnSendResponse( pfc, (HTTP_FILTER_SEND_RESPONSE*)pvNotification );
		} else if( notificationType == SF_NOTIFY_SEND_RAW_DATA ) {
			return OnSendRawData( pfc, (HTTP_FILTER_RAW_DATA*)pvNotification );
		} else if (notificationType == SF_NOTIFY_END_OF_REQUEST )  {
			return OnEndOfRequest( pfc ); 
		} else {
			return SF_STATUS_REQ_NEXT_NOTIFICATION;
		} // else
	} catch( ... ) {
		TRACE0( "[MIF] ERROR: unhandled exception in HttpFilterProc\n" );
		DisableNotifications(pfc); 
		return SF_STATUS_REQ_NEXT_NOTIFICATION;
	} // catch
}




// ----------------------------------------------------------------------------
// Called when IIS unloads our filter. This only happens when IIS itself
// terminates
// ----------------------------------------------------------------------------
BOOL WINAPI TerminateFilter(
	DWORD /*dwFlags*/
)
{
	try {
		SaveFilterData("");
	} catch( ... ) {
		TRACE0( "[MIF] ERROR: unhandled exception in TerminateFilter\n" );
	} // catch

	return TRUE;
}

 

// ----------------------------------------------------------------------------
// DLL entry point
// ----------------------------------------------------------------------------
BOOL APIENTRY DllMain( HANDLE /*hModule*/, 
                       DWORD  /*ul_reason_for_call*/, 
                       LPVOID /*lpReserved*/
					 )
{
    return TRUE;
}


