#include "stdafx.h"
#pragma warning( disable : 4786 )  
#include "PageInfo.h"


// ----------------------------------------------------------------------------
// ----------------------------------------------------------------------------
void PageMap::Reset()
{
	CritSectLock lock(&cs); // lock pagemap
	for( INTERNALPAGEMAP::iterator it = map.begin(); it != map.end(); it++ ) {
		PageInfo* pPageInfo = it->second;
		delete pPageInfo;
	} // for
	map.clear();
}
 

// ----------------------------------------------------------------------------
// Fetches PageInfo pointer from map
// ----------------------------------------------------------------------------
PageInfo* PageMap::GetPageInfo( LPCSTR lpszPageName ) 
{
	if( ! (lpszPageName && *lpszPageName)) {
		return NULL;
	} // if

	CritSectLock lock(&cs); // lock pagemap
	INTERNALPAGEMAP::iterator it = map.find( lpszPageName );
	if( it != map.end() ) {
		return it->second; 
	} else {
		return NULL; 
	} // else
}


// ----------------------------------------------------------------------------
// Fetches PageInfo pointer from map. Creates it if it doesn't exist
// ----------------------------------------------------------------------------
PageInfo* PageMap::GetOrCreatePageInfo( LPCSTR lpszPageName ) 
{
	if( ! (lpszPageName && *lpszPageName)) {
		return NULL;
	} // if

	CritSectLock lock(&cs); // lock pagemap
	PageInfo* pPageInfo = GetPageInfo( lpszPageName ); 
	if( pPageInfo == NULL ) {
		pPageInfo = new PageInfo(); 
		map.insert(pair<string,PageInfo*>( lpszPageName, pPageInfo )); 
	} 
	return pPageInfo; 
}

// ----------------------------------------------------------------------------
// Dumps information to a stream in tab-delimited format
// ----------------------------------------------------------------------------
void PageMap::Dump( strstream& stream ) 
{
	const int BUFF_SIZE = 1024;
	char szLine[BUFF_SIZE]; 
	CritSectLock lock(&cs); // lock pagemap
	for( INTERNALPAGEMAP::iterator it = map.begin(); it != map.end(); it++ ) {
		PageInfo* pPageInfo = it->second;
		Counter* pDuration = &(pPageInfo->duration);
		Counter* pBytes = &(pPageInfo->bytes);
		DWORD avgDuration = pDuration->Average( pPageInfo->numcalls );
		DWORD avgBytes = pBytes->Average( pPageInfo->numcalls );
		DWORD avgBytesPerSec = (avgBytes * 1000) / avgDuration;

		ZeroMemory( szLine, BUFF_SIZE ); 
		_snprintf( szLine, BUFF_SIZE-1, 
			"%s\t%lu\t%lu\t%lu\t%lu\t%lu\t%lu\t%lu\t%lu\t%lu\t%lu\n", 
			it->first.c_str(), pPageInfo->numcalls, 
			pDuration->dMin, pDuration->dMax, pDuration->accum, avgDuration,
			pBytes->dMin, pBytes->dMax, pBytes->accum, avgBytes, avgBytesPerSec );

		stream << szLine; 
	} // for
}

// ----------------------------------------------------------------------------
// Loads information from one line of tab-delimited string as saved by Dump
// ----------------------------------------------------------------------------
BOOL PageMap::LoadLine( LPSTR lpszLine ) 
{
	CritSectLock lock(&cs); // lock pagemap

	const char DELIMS[] = "\t\n"; 
	char* szPageName = NULL;
	char* token = strtok( lpszLine, DELIMS ); 

	if( ! (token && *token) ) {
		return FALSE;
	} // if

	szPageName = token; 
	// 10 values should follow
	DWORD value[10]; 
	for( int i=0; i<8; i++ ) {
		token = strtok( NULL, DELIMS ); 
		if( ! (token && *token && isdigit(*token))) {
			return FALSE; 
		} // if
		value[i] = atol(token);
	} // for

	// If we get here, we parsed the line completely
	PageInfo* pPageInfo = GetOrCreatePageInfo( szPageName );
	if( pPageInfo ) {
		CritSectLock lock(&(pPageInfo->cs)); // lock PageInfo structure (overkill)
		pPageInfo->numcalls = value[0];

		pPageInfo->duration.dMin = value[1];
		pPageInfo->duration.dMax = value[2];
		// restore accumulated duration from average duration
		pPageInfo->duration.accum = value[3]; 
		// value[4] not used

		pPageInfo->bytes.dMin = value[5];
		pPageInfo->bytes.dMax = value[6];
		// restore accumulated bytes from average bytes
		pPageInfo->bytes.accum = value[7]; 

		// value[8] and value[9] not used
	} // if

	return TRUE; 
}

