// ----------------------------------------------------------------------------
// ----------------------------------------------------------------------------
// Timer.cpp
// ----------------------------------------------------------------------------
// ----------------------------------------------------------------------------
#include "stdafx.h"
#pragma warning( disable : 4786 )  
#include "Timer.h"
#include <stdio.h>

// ----------------------------------------------------------------------------
// _HiResCounter
// ----------------------------------------------------------------------------
class _HiResCounter{
public:
	_HiResCounter()
		{ m_bExists = QueryPerformanceFrequency( &m_Freq ); }

	BOOL Exists()
		{ return m_bExists; }

	double Freq()
		{ return (double)(m_Freq.QuadPart); }

protected:
	BOOL m_bExists; 
	LARGE_INTEGER m_Freq;
}; 

static _HiResCounter HiResCounter; 


// ----------------------------------------------------------------------------
// Returns TRUE if high-resolution multimedia timer is installed. Most modern
// motherboards have one.
// ----------------------------------------------------------------------------
BOOL IsHighResolutionTimerAvailable()
{
	return ( HiResCounter.Exists() );
}


// ----------------------------------------------------------------------------
// Begins a timing block. Pass the return value to EndTiming() to get total
// elapsed time.
// ----------------------------------------------------------------------------
double StartTiming()
{
	LARGE_INTEGER li;  
	if( HiResCounter.Exists() ) {
		QueryPerformanceCounter( &li ); 
		return (double)(li.QuadPart);
	} else {
		return (double)GetTickCount(); 
	} // else
}


// ----------------------------------------------------------------------------
// Returns elapsed time based on dStart, which is the return value of 
// StartTiming(). If a multimedia timer is available, the result is accurate
// to the precision of the multimedia timer. Otherwise, the result is as
// accurate as GetTickCount()
// ----------------------------------------------------------------------------
double EndTiming( double dStart )
{
	LARGE_INTEGER li; 
	if( HiResCounter.Exists() ) {
		QueryPerformanceCounter( &li ); 
		return ((li.QuadPart - dStart) * 1000.0 / HiResCounter.Freq());
	} else {
		return (GetTickCount() - (DWORD) dStart); 
	} // else

}


// ----------------------------------------------------------------------------
// ----------------------------------------------------------------------------
CProfileScope::~CProfileScope()
{
	double dElapsed = ::EndTiming(m_dStart);
	const int BUFF_SIZE=256;
	CHAR buff[BUFF_SIZE];
	if( _snprintf( buff, BUFF_SIZE, "%s: %1.2fms", m_strName.c_str(), dElapsed) < 0 ) {
		buff[BUFF_SIZE-1] = '\0';
	} // if
	::OutputDebugStringA(buff);
}

